#!/bin/bash
set -e

DOCKER_IMAGE_NAME=iopsys/peripheral_manager
NAME=peripheral_manager
WORKDIR=/opt/work
DOCKER_FOLDER=$(dirname $0)
DOCKER_PORT=22
HOST_PORT=2222
LOCAL_USER_ID=`id -u $USER`

help() {
    echo "This build_docker script to minimize the effort of running docker command all the time"
    echo "  $0 <command>"
    echo "     clean       Cleans all hanging images or not labelled images "
    echo "     cleanall    clean + removes the build image"     
    echo "     build       builds the docker image"
    echo "     service     run the service container of ubusd daemon" 
    echo "     run [args]  runs image, with user command to execute on docker container"
    echo "    #################################################################### "
    echo "     ******    Steps to run the Docker Container                   ****** "
    echo "     ******      ./build_docker.sh build                           ****** "
    echo "     ******      ./build_docker.sh service  (In one terminal)      ****** "
    echo "     ******      ./build_docker.sh run bash (In another terminal)  ****** "
    echo "     ******                OR                                      ****** "
    echo "     ****** ./build_docker.sh run cmake -DCMAKE_BUILD_TYPE=Debug . ****** "
    echo "     ****** ./build_docker.sh run make (Builds the binary of UT)   ****** "
    echo "    #################################################################### "
}

clean() {
    echo Removing
    docker image prune -f
    docker container prune -f
}

cleanall() {
    clean
    docker rmi --force $DOCKER_IMAGE_NAME || exit $?
}

stop() {
    docker stop $NAME
}

check_image() {
    docker images $DOCKER_IMAGE_NAME
}

check_docker() {
     check_docker=$(docker -v)
     echo "$check_docker"
     if [[ $check_docker =~ .*"Docker version"* ]]; then
       echo "Docker is present and its starting to build the container"
     else
       echo "*** Please Install Docker, For reference Use the below link ******"
       echo "*** https://docs.docker.com/engine/install/ubuntu/ *******"
       exit 0
     fi
}

build() {
     
     check_docker
     docker build -t $DOCKER_IMAGE_NAME $DOCKER_FOLDER $*
}

run() {
	
      if [ -z "$*" ]; then
        args="bash"
      else
        args=$*
      fi

      echo Running docker container with args: $args
      docker exec -it $NAME $args
      
}

service() {
    echo Running docker service container with args: $args
    cmd="docker run -it --privileged --rm
        --name=$NAME
        --ipc=host
	 -e $LOCAL_USER_ID
        --volume $(pwd):$WORKDIR
        -p $HOST_PORT:$DOCKER_PORT
        -w $WORKDIR
        $DOCKER_IMAGE_NAME "
    $cmd
}

if [ -z "$*" ]; then
    help
else
    cmd="$1"
    shift
    $cmd $*
fi
