#include <syslog.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include "gpio.h"
#include "log.h"

static void gpio_linux_dir_init(int addr, int dir ){
	char buf[128] = {0};
	int fd = -1;

	/* export gpio to userspace */
	snprintf(buf, sizeof(buf), "%d\n", addr);
	fd = open("/sys/class/gpio/export", O_WRONLY);
	write(fd, buf, strlen(buf));
	close(fd);

	/* set gpio as output */
	snprintf(buf, sizeof(buf), "/sys/class/gpio/gpio%d/direction",addr);
	fd = open(buf, O_WRONLY);
	if (dir) {
		snprintf(buf, sizeof(buf), "in\n");
	} else {
		snprintf(buf, sizeof(buf), "out\n");
	}
	write(fd, buf, strlen(buf));
	close(fd);
}


void gpio_linux_output_init(int addr){
	gpio_linux_dir_init(addr, 0);
}

void gpio_linux_input_init(int addr){
	gpio_linux_dir_init(addr, 1);
}

int gpio_linux_set(int addr, int val)
{
	char buf[128] = {0};
	int fd = -1;

	snprintf(buf, sizeof(buf), "/sys/class/gpio/gpio%d/value", addr);
	fd = open(buf, O_WRONLY);

	snprintf(buf, sizeof(buf), "%d\n", val);

	write(fd, buf, strlen(buf));
	close(fd);

	return 0;
}

int gpio_linux_get(int addr)
{
	char buf[128] = {0};
	int fd = -1;
	int size = 0;

	snprintf(buf, sizeof(buf), "/sys/class/gpio/gpio%d/value", addr);
	fd = open(buf, O_RDONLY);

	size = read(fd, buf, 100);
	close(fd);

	if(size){
		return strtol(buf, 0, 0);
	}
	return 0;
}


gpio_state_t gpio_get_state(gpio_t gpio)
{
	return gpio_linux_get(gpio);
}

void gpio_set_state(gpio_t gpio, gpio_state_t state, int udelay)
{
	gpio_linux_output_init(gpio);
	gpio_linux_set(gpio, state);

	if (udelay)
		usleep(udelay);

	return;
}
